;(function () {

    if (window.WebViewJavascript) {
        return;
    }

    var messageHandlers = {};
    var responseCallbacks = {};
    var uniqueId = 1;

    function init(messageHandler) {
        if (WebViewJavascriptBridge._messageHandler) {
            throw new Error('WebViewJavascriptBridge.init called twice');
        }
        WebViewJavascriptBridge._messageHandler = messageHandler;
    }

    function send(data, responseCallback) {
        _doSend({data: data}, responseCallback);
    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function callHandler(handlerName, data, responseCallback) {
        if (typeof data === 'function') {
            _doSend({handlerName: handlerName, data: ""}, data);
        } else {
            _doSend({handlerName: handlerName, data: data}, responseCallback);
        }
    }

    function _doSend(message, responseCallback) {
        console.log("responseCallback:" + responseCallback);
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + "_" + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message['callbackId'] = callbackId
        }
        console.log("sending:" + JSON.stringify(message));
        var msg_data = (typeof message.data === 'string')
        && message.data.constructor === String
            ? message.data : JSON.stringify(message.data) || "";
        var msg_responseId = message.responseId || "";
        var msg_responseData = message.responseData || "";
        var msg_callbackId = message.callbackId || "";
        var msg_handlerName = message.handlerName || "";

        _WebViewJavascriptBridge._handleMessageFromJs(
            msg_data,
            msg_responseId,
            msg_responseData,
            msg_callbackId,
            msg_handlerName);

    }

    function _dispatchMessageFromJava(messageJSON) {
        var message = JSON.parse(messageJSON);
        var messageHandler;
        if (message.responseId) {
            var responseCallback = responseCallbacks[message.responseId];
            if (!responseCallback) {
                return
            }
            responseCallback((typeof message.responseData === 'string')
            && message.responseData.constructor === String ?
                message.responseData : JSON.parse(message.responseData));
            delete responseCallbacks[message.responseId];
        } else {
            var responseCallback;
            if (message.callbackId) {
                var callbackResponseId = message.callbackId;
                responseCallback = function (responseData) {
                    _doSend({responseId: callbackResponseId, responseData: responseData});
                }
            }
            var handler = WebViewJavascriptBridge._messageHandler;
            if (message.handlerName) {
                handler = messageHandlers[message.handlerName];
            }
            try {
                handler(message.data, responseCallback);
            } catch (exception) {
                if (typeof console !== 'undefined') {
                    console.log('WebViewJavascriptBridge: WARNING: javascript handler threw.', message, exception);
                }
            }
        }
    }

    function _handleMessageFromJava(messageJSON) {
        _dispatchMessageFromJava(messageJSON);
    }

    window.WebViewJavascriptBridge = {
        init: init,
        send: send,
        registerHandler: registerHandler,
        callHandler: callHandler,
        _handleMessageFromJava: _handleMessageFromJava
    };

    var doc = document;
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('WebViewJavascriptBridgeReady');
    readyEvent.bridge = WebViewJavascriptBridge;
    doc.dispatchEvent(readyEvent);
})();